import { ViewBase, View, ContainerView, EventData, StackLayout, Property, KeyedTemplate, Color } from '@nativescript/core';
export declare const NG_VIEW = "ng_view";
export declare enum ListViewViewType {
    HeaderView = "HeaderView",
    ItemView = "ItemView",
    FooterView = "FooterView",
    GroupView = "GroupView",
    LoadOnDemandView = "LoadOnDemandView",
    ItemSwipeView = "ItemSwipeView"
}
export declare enum ListViewItemSnapMode {
    Auto = "Auto",
    Start = "Start",
    End = "End",
    Center = "Center"
}
export declare enum ListViewScrollDirection {
    Vertical = "Vertical",
    Horizontal = "Horizontal"
}
export declare enum ListViewScrollPosition {
    None = "None",
    Top = "Top",
    CenteredVertically = "CenteredVertically",
    CenteredHorizontally = "CenteredHorizontally",
    Bottom = "Bottom",
    Left = "Left",
    Right = "Right"
}
export declare enum ListViewReorderMode {
    HoldAndDrag = "HoldAndDrag",
    Drag = "Drag"
}
export declare enum ListViewItemAnimation {
    Default = "Default",
    Fade = "Fade",
    Scale = "Scale",
    Slide = "Slide"
}
export declare enum ListViewLoadOnDemandMode {
    None = "None",
    Manual = "Manual",
    Auto = "Auto"
}
export declare enum ListViewSelectionBehavior {
    None = "None",
    Press = "Press",
    LongPress = "LongPress"
}
export declare class PullToRefreshStyle extends ViewBase {
    indicatorColor: Color;
    indicatorBackgroundColor: Color;
    _owner: RadListView;
    static indicatorColorProperty: Property<PullToRefreshStyle, Color>;
    static indicatorBackgroundColorProperty: Property<PullToRefreshStyle, Color>;
    protected onIndicatorColorPropertyChanged(oldValue: Color, newValue: Color): void;
    protected onIndicatorBackgroundColorPropertyChanged(oldValue: Color, newValue: Color): void;
}
export declare class ReorderHandle extends StackLayout {
    constructor();
}
export declare class ListViewScrollEventData implements EventData {
    private _eventName;
    private _object;
    private _scrollOffset;
    constructor();
    get object(): any;
    set object(value: any);
    set eventName(value: string);
    get eventName(): string;
    get scrollOffset(): number;
    set scrollOffset(value: number);
}
export declare class ListViewEventData implements EventData {
    private _eventName;
    private _object;
    private _index;
    private _dataItemIndex;
    private _groupIndex;
    private _data;
    private _returnValue;
    private _view;
    private _android;
    private _ios;
    constructor();
    get android(): any;
    set android(value: any);
    get ios(): any;
    set ios(value: any);
    get eventName(): string;
    set eventName(value: string);
    get object(): any;
    set object(value: any);
    get index(): number;
    set index(value: number);
    get dataItemIndex(): number;
    set dataItemIndex(value: number);
    get groupIndex(): number;
    set groupIndex(value: number);
    get data(): any;
    set data(value: any);
    get returnValue(): any;
    set returnValue(value: any);
    get view(): View;
    set view(value: View);
}
export declare class LoadOnDemandListViewEventData extends ListViewEventData {
}
export declare class SwipeActionsEventData extends ListViewEventData {
    private _swipeView;
    private _mainView;
    get mainView(): any;
    set mainView(value: any);
    get swipeView(): any;
    set swipeView(value: any);
}
export declare class ListViewLayoutBase extends ViewBase {
    scrollDirection: ListViewScrollDirection;
    itemInsertAnimation: ListViewItemAnimation;
    itemDeleteAnimation: ListViewItemAnimation;
    itemWidth: number;
    itemHeight: number;
    get android(): any;
    get ios(): any;
    static scrollDirectionProperty: Property<ListViewLayoutBase, ListViewScrollDirection>;
    private onScrollDirectionPropertyChanged;
    protected onScrollDirectionChanged(oldValue: ListViewScrollDirection, newValue: ListViewScrollDirection): void;
    static itemInsertAnimationProperty: Property<ListViewLayoutBase, ListViewItemAnimation>;
    private onItemInsertAnimationPropertyChanged;
    protected onItemInsertAnimationChanged(oldValue: ListViewItemAnimation, newValue: ListViewItemAnimation): void;
    static itemDeleteAnimationProperty: Property<ListViewLayoutBase, ListViewItemAnimation>;
    private onItemDeleteAnimationPropertyChanged;
    protected onItemDeleteAnimationChanged(oldValue: ListViewItemAnimation, newValue: ListViewItemAnimation): void;
    static itemWidthProperty: Property<ListViewLayoutBase, number>;
    private onItemWidthPropertyChanged;
    protected onItemWidthChanged(oldValue: number, newValue: number): void;
    static itemHeightProperty: Property<ListViewLayoutBase, number>;
    private onItemHeightPropertyChanged;
    protected onItemHeightChanged(oldValue: number, newValue: number): void;
    _onOwnerUICreated(): void;
}
export declare class RadListView extends ContainerView {
    static knownFunctions: string[];
    static scrolledEvent: string;
    static scrollDragEndedEvent: string;
    static scrollStartedEvent: string;
    static scrollEndedEvent: string;
    static itemSelectingEvent: string;
    static itemDeselectingEvent: string;
    static itemTapEvent: string;
    static itemHoldEvent: string;
    static itemSelectedEvent: string;
    static itemDeselectedEvent: string;
    static itemReorderStartingEvent: string;
    static itemReorderedEvent: string;
    static itemReorderStartedEvent: string;
    static itemSwipingEvent: string;
    static itemSwipeProgressChangedEvent: string;
    static itemSwipeProgressStartedEvent: string;
    static itemSwipeProgressEndedEvent: string;
    static loadMoreDataRequestedEvent: string;
    static pullToRefreshInitiatedEvent: string;
    static itemLoadingEvent: string;
    static itemLoadingInternalEvent: string;
    static dataPopulatedEvent: string;
    pullToRefreshStyle: PullToRefreshStyle;
    listViewLayout: ListViewLayoutBase;
    headerItemTemplate: string;
    _hasGroupingFunctionChanged: boolean;
    footerItemTemplate: string;
    itemTemplate: string;
    groupTemplate: string;
    itemTemplates: string | Array<KeyedTemplate>;
    itemTemplateSelector: ((item: any, index: number, items: any) => string) | string;
    itemSwipeTemplate: string;
    multipleSelection: boolean;
    itemReorder: boolean;
    itemSwipe: boolean;
    swipeActions: boolean;
    pullToRefresh: boolean;
    loadOnDemandMode: ListViewLoadOnDemandMode;
    loadOnDemandBufferSize: number;
    selectionBehavior: ListViewSelectionBehavior;
    items: any;
    scrollPosition: ListViewScrollPosition;
    reorderMode: ListViewReorderMode;
    loadOnDemandItemTemplate: string;
    enableCollapsibleGroups: boolean;
    groupingFunction: (item: any) => any;
    filteringFunction: (item: any) => boolean;
    sortingFunction: (item: any, otherItem: any) => number;
    _loadOnDemandModeInternal: ListViewLoadOnDemandMode;
    itemSelectedBackgroundColor: Color;
    private _isEventListenerAttached;
    _defaultTemplate: KeyedTemplate;
    onLoaded(): void;
    private logWarningIfNeeded;
    onUnloaded(): void;
    _itemTemplatesInternal: KeyedTemplate[];
    protected _dataUpdatesSuspended: boolean;
    private _itemViewLoader;
    private _itemViewDisposer;
    private _nativeScriptViewAdded;
    private _itemTemplateSelectorBindable;
    get itemViewLoader(): (viewType: any) => View;
    get itemViewDisposer(): () => void;
    get nativeScriptViewAdded(): (parent: View, child: View) => void;
    get isDataOperationsEnabled(): boolean;
    set itemViewLoader(value: (viewType: any) => View);
    set itemViewDisposer(value: () => void);
    set nativeScriptViewAdded(value: (parent: View, child: View) => void);
    static itemSelectedBackgroundColorProperty: Property<RadListView, Color>;
    static groupingFunctionProperty: Property<RadListView, (item: any) => any>;
    static filteringFunctionProperty: Property<RadListView, (item: any) => boolean>;
    static sortingFunctionProperty: Property<RadListView, (item: any, otherItem: any) => number>;
    static enableCollapsibleGroupsProperty: Property<RadListView, boolean>;
    static pullToRefreshStyleProperty: Property<RadListView, PullToRefreshStyle>;
    static headerItemTemplateProperty: Property<RadListView, string>;
    static footerItemTemplateProperty: Property<RadListView, string>;
    static reorderModeProperty: Property<RadListView, ListViewReorderMode>;
    static listViewLayoutProperty: Property<RadListView, ListViewLayoutBase>;
    static loadOnDemandItemTemplateProperty: Property<RadListView, string>;
    static itemTemplateSelectorProperty: Property<RadListView, string | ((item: any, index: number, items: any) => string)>;
    static itemTemplatesProperty: Property<RadListView, string | KeyedTemplate[]>;
    static itemTemplateProperty: Property<RadListView, string>;
    static itemSwipeTemplateProperty: Property<RadListView, string>;
    static groupTemplateProperty: Property<RadListView, string>;
    static multipleSelectionProperty: Property<RadListView, boolean>;
    static itemReorderProperty: Property<RadListView, boolean>;
    static itemSwipeProperty: Property<RadListView, boolean>;
    static swipeActionsProperty: Property<RadListView, boolean>;
    static pullToRefreshProperty: Property<RadListView, boolean>;
    static loadOnDemandModeProperty: Property<RadListView, ListViewLoadOnDemandMode>;
    static loadOnDemandBufferSizeProperty: Property<RadListView, number>;
    static selectionBehaviorProperty: Property<RadListView, ListViewSelectionBehavior>;
    static itemsProperty: Property<RadListView, any>;
    static scrollPositionProperty: Property<RadListView, ListViewScrollPosition>;
    _reorderItemInSource(oldPosition: number, newPosition: number): void;
    suspendUpdates(): void;
    resumeUpdates(refresh: boolean): void;
    updatesSuspended(): boolean;
    getItemAtIndex(index: any): any;
    getIndexOf(item: any): any;
    selectItemAt(index: number): void;
    deselectItemAt(index: number): void;
    selectAll(): void;
    deselectAll(): void;
    isItemSelected(item: any): boolean;
    getSelectedItems(): any[];
    getViewForItem(item: any): View;
    protected resolveTemplateView(template: string): View;
    protected _getDefaultGroupContent(): View;
    getViewForViewType(viewType: any, templateKey?: string): View;
    private onItemSelectedBackgroundColorPropertyChanged;
    private onGroupingFunctionPropertyChanged;
    private onFilteringFunctionPropertyChanged;
    private onSortingFunctionPropertyChanged;
    private onEnableCollapsibleGroupsPropertyChanged;
    private _getDefaultItemContent;
    private getTemplateFromSelector;
    private onPullToRefreshStylePropertyChanged;
    private onHeaderItemTemplatePropertyChanged;
    private onFooterItemTemplatePropertyChanged;
    private onLoadOnDemandItemTemplatePropertyChanged;
    private onListViewReorderModePropertyChanged;
    private onLayoutPropertyChanged;
    private onItemTemplateSelectorPropertyChanged;
    private onItemTemplatesPropertyChanged;
    private onItemTemplatePropertyChanged;
    private onGroupTemplatePropertyChanged;
    private onItemSwipeTemplatePropertyChanged;
    private onMultipleSelectionPropertyChanged;
    private onItemReorderPropertyChanged;
    private onItemSwipePropertyChanged;
    private onSwipeActionsPropertyChanged;
    private onPullToRefreshPropertyChanged;
    private onLoadOnDemandModePropertyChanged;
    private onLoadOnDemandBufferSizePropertyChanged;
    private onSelectionBehaviorPropertyChanged;
    private onItemsPropertyChanged;
    private onScrollPositionPropertyChanged;
    protected onItemViewLoaderChanged(): void;
    protected onItemViewDisposerChanged(): void;
    protected onNativeScriptViewAddedChanged(): void;
    protected onItemSelectedBackgroundColorChanged(oldValue: Color, newValue: Color): void;
    protected onGroupingFunctionChanged(oldValue: (item: any) => any, newValue: (item: any) => any): void;
    protected onFilteringFunctionChanged(oldValue: (item: any) => boolean, newValue: (item: any) => boolean): void;
    protected onSortingFunctionChanged(oldValue: (item: any, otherItem: any) => number, newValue: (item: any, otherItem: any) => number): void;
    protected onEnableCollapsibleGroupsChanged(oldValue: boolean, newValue: boolean): void;
    protected onPullToRefreshStyleChanged(oldValue: PullToRefreshStyle, newValue: PullToRefreshStyle): void;
    protected onHeaderItemTemplateChanged(oldValue: string, newValue: string): void;
    protected onFooterItemTemplateChanged(oldValue: string, newValue: string): void;
    protected onLoadOnDemandItemTemplateChanged(oldValue: string, newValue: string): void;
    protected onReorderModeChanged(oldValue: ListViewReorderMode, newValue: ListViewReorderMode): void;
    protected onListViewLayoutChanged(oldValue: ListViewLayoutBase, newValue: ListViewLayoutBase): void;
    protected onItemTemplateSelectorChanged(oldValue: string | ((item: any, index: number, items: any) => string), newValue: string | ((item: any, index: number, items: any) => string)): void;
    protected onItemTemplatesChanged(oldValue: string | Array<KeyedTemplate>, newValue: string | Array<KeyedTemplate>): void;
    protected onItemTemplateChanged(oldValue: string, newValue: string): void;
    protected onGroupTemplateChanged(oldValue: string, newValue: string): void;
    protected onItemSwipeTemplateChanged(oldValue: string, newValue: string): void;
    protected onMultipleSelectionChanged(oldValue: boolean, newValue: boolean): void;
    protected onItemReorderChanged(oldValue: boolean, newValue: boolean): void;
    protected onItemSwipeChanged(oldValue: boolean, newValue: boolean): void;
    protected onSwipeActionsChanged(oldValue: boolean, newValue: boolean): void;
    protected onPullToRefreshChanged(oldValue: boolean, newValue: boolean): void;
    protected onLoadOnDemandModeChanged(oldValue: ListViewLoadOnDemandMode, newValue: ListViewLoadOnDemandMode): void;
    protected onLoadOnDemandBufferSizeChanged(oldValue: number, newValue: number): void;
    protected onSelectionBehaviorChanged(oldValue: ListViewSelectionBehavior, newValue: ListViewSelectionBehavior): void;
    private onSourceCollectionChangedInternal;
    private onItemsChangedInternal;
    protected onItemsChanged(oldValue: any, newValue: any): void;
    protected onScrollPositionChanged(oldValue: ListViewScrollPosition, newValue: ListViewScrollPosition): void;
    protected clearEmbeddedViews(): void;
    protected onSourceCollectionChanged(data: any): void;
    refresh(): void;
    updateHeaderFooter(): void;
    getScrollOffset(): number;
    scrollToIndex(index: number, animate?: boolean, snapMode?: ListViewItemSnapMode): void;
    scrollWithAmount(amount: number, animate: boolean): void;
    notifyLoadOnDemandFinished(disableLoadOnDemand?: boolean): void;
    notifyPullToRefreshFinished(): void;
    notifySwipeToExecuteFinished(): void;
}
